let events = {};
let id = 1;

let bindMap = {
    "appStarted":'start',
    "viewChanged":"route"
}

let triggedMap = {};
/**
 * 绑定自定义事件
 * @param event 自定义事件
 * @param handler 自定义事件的处理函数
 */
exports.bind = function (event, handler) {
    // appStarted，viewChanged
    if(bindMap[event]) event = bindMap[event];

    handler.id = id;
    events[event] = events[event] || [];
    events[event].push(handler);
    if(triggedMap[event]){
        handler.apply({
            event: event
        }, triggedMap[event]);
    }
    return id++;
};

/**
 * 主动触发应用级事件
 */
exports.trigger = function (event,...params) {
    triggedMap[event] = params;
    let triggerPass = "true";
    if (events[event] && events[event].length > 0) {
        events[event].forEach(function (handler) {
            //事件的处理必须是异步的
            let doHandler = function () {
                handler.apply({
                    event: event
                }, params);
            };
            //154941 light框架支持Light.on("viewback") 
            if(params[0]&&params[0].sync){
                let triggerResult = handler.apply({
                    event: event
                }, params);
                if(!triggerResult){
                    triggerPass = "false";
                }
            }else{
                setTimeout(doHandler,0);
            }
            
        });
    }
    return triggerPass;
};

/**
 * 解绑事件
 */
exports.unbind = function (event,id) {
    if(id&&events[event]){
        events[event] = events[event].filter(function (handler) {
            return handler.id !== id
        })
    }else{
        events[event] = [];
    }
};